{% extends "course_flow/base.html" %}
<!-- -->
{% load static i18n %}
<!-- -->
{% block header %} {% endblock %}
<!-- -->
{% block title %}{% trans "My Classrooms" %} - CourseFlow{% endblock %}
<!-- -->
{% block metadescription %} {% endblock %}
<!-- -->
{% block body %}
<!-- -->
{% endblock %}
<!-- -->
{% block foot %} {% endblock %}
<!-- -->
{% csrf_token %}
<!-- -->
{% block scripts %} {{ block.super }}
<script nonce="{{request.csp_nonce}}">

  const data_package = {{project_data_package|safe}}

  var home_renderer;
  const user_id = {{ user.id }};
  const read_only=false;
  window.addEventListener("load",function(){
      home_renderer = new renderers.WorkflowGridRenderer(data_package);
      $(document).ajaxError(renderers.fail_function);
      home_renderer.render($("#container"));
      makeActiveSidebar("#panel-my-live-projects");
  });


</script>
{% endblock %}
