# Generated by Django 3.2.15 on 2023-04-09 14:12

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("course_flow", "0111_alter_favourite_content_type"),
    ]

    operations = [
        migrations.AddField(
            model_name="column",
            name="icon",
            field=models.CharField(blank=True, max_length=50, null=True),
        ),
    ]
