{% extends "course_flow/base.html" %}

{% load static i18n %} {% load course_flow_templatetags %}

{% block title %}{% trans "Home" %} - CourseFlow{% endblock %}

{% csrf_token %}

{% block scripts %}
  {{ block.super }}
  <script src="{% static 'course_flow/js/templates/home.js' %}"></script>
{% endblock %}
