{% load static compress  %}
<!-- External resources -->
    <!-- * Fonts and icons -->

    <!-- * Scripts -->
    <!-- Matomo -->
    <!-- <script>
      var _paq = window._paq = window._paq || [];
      /* tracker methods like "setCustomDimension" should be called before "trackPageView" */
      _paq.push(['trackPageView']);
      _paq.push(['enableLinkTracking']);
      (function() {
        var u="//matomo-analytics-staging.mydalite.org/";
        _paq.push(['setTrackerUrl', u+'matomo.php']);
        _paq.push(['setSiteId', '2']);
        var d=document, g=d.createElement('script'), s=d.getElementsByTagName('script')[0];
        g.async=true; g.src=u+'matomo.js'; s.parentNode.insertBefore(g,s);
      })();
    </script> -->
    <!-- End Matomo Code -->

    <!-- ** Preloads -->
    <!-- ** Polyfills for non-awesome browsers-->
    <script
      src="https://cdn.polyfill.io/v2/polyfill.min.js"
      rel="preload"
      integrity="sha384-83CYOGXsi59YYxBeS/ag/t615snxzc9UUt0t4ApSQHHKYmbmlQj/MuS9QywJHVDN"
      crossorigin="anonymous"
    ></script>
    <!-- ** jquery -->
    <script
      src="https://ajax.googleapis.com/ajax/libs/jquery/3.4.1/jquery.min.js"
      integrity="sha384-vk5WoKIaW/vJyUAd9n/wmopsmNhiy+L2Z+SBxGYnUkunIxVxAv/UtMOhba/xskxh"
      crossorigin="anonymous"
    ></script>
    <script
      src="https://ajax.googleapis.com/ajax/libs/jqueryui/1.12.1/jquery-ui.min.js"
      integrity="sha384-Dziy8F2VlJQLMShA6FHWNul/veM9bCkRUaLqr199K94ntO5QUrLJBEbYegdSkkqX"
      crossorigin="anonymous"
    ></script>
    <!-- ** d3 -->
    <script
      src="https://d3js.org/d3.v5.min.js"
      integrity="sha384-M06Cb6r/Yrkprjr7ngOrJlzgekrkkdmGZDES/SUnkUpUol0/qjsaQWQfLzq0mcfg"
      crossorigin="anonymous"
    ></script>
    <!-- Quill Stylesheet-->
    <link
      rel="stylesheet"
      href="https://cdn.quilljs.com/1.3.6/quill.snow.css"
      integrity="sha384-07UbSXbd8HpaOfxZsiO6Y8H1HTX6v0J96b5qP6PKSpYEuSZSYD4GFFHlLRjvjVrL"
      crossorigin="anonymous"
    />
    <!-- Include the Quill library -->
    <script
      src="https://cdn.quilljs.com/1.3.6/quill.min.js"
      integrity="sha384-AOnYUgW6MQR78EvTqtkbNavz7dVI7dtLm7QdcIMBoy06adLIzNT40hHPYl9Rr5m5"
      crossorigin="anonymous"
    ></script>
    <!-- fonts -->
    <link href='//fonts.googleapis.com/css?family=Open+Sans:400italic,600italic,700italic,400,300,600,700' rel='stylesheet' type='text/css'>
    <link rel="stylesheet" href="https://fonts.googleapis.com/css2?family=Material+Symbols+Rounded:opsz,wght,FILL,GRAD@20..48,100..700,0..1,-50..200" />
    <link rel="stylesheet" href="https://fonts.googleapis.com/css2?family=Material+Symbols+Outlined:opsz,wght,FILL,GRAD@20..48,100..700,0..1,-50..200" />
    <!-- * Icons -->
    <link rel="icon" type="image/x-icon" href="{% static 'course_flow/img/courseflow-favicon.png' %}";>
    <!-- ** Preloads -->
    <link
      rel="preload"
      href="https://fonts.googleapis.com/icon?family=Material+Icons+Rounded&display=swap"
      as="style"
    />
    <link
      rel="preload"
      href="https://fonts.googleapis.com/icon?family=Material+Icons+Outlined&display=swap"
      as="style"
    />
    <!-- ** Loads -->
    <link
      rel="stylesheet"
      href="https://fonts.googleapis.com/icon?family=Material+Icons+Rounded&display=swap"
      type="text/css"
    />
